from kvdroid import activity


def dark_mode():
    from kvdroid.jclass.android import Configuration
    Configuration = Configuration()
    night_mode_flags = activity.getContext().getResources().getConfiguration().uiMode & Configuration.UI_MODE_NIGHT_MASK
    if night_mode_flags == Configuration.UI_MODE_NIGHT_YES:
        return True
    elif night_mode_flags in [
        Configuration.UI_MODE_NIGHT_NO,
        Configuration.UI_MODE_NIGHT_UNDEFINED,
    ]:
        return False



from jnius import autoclass
from kvdroid.jclass import _class_call


def Uri(*args, instantiate: bool = False):
    return _class_call(autoclass('android.net.Uri'), args, instantiate)


def ConnectivityManager(*args, instantiate: bool = False):
    return _class_call(autoclass("android.net.ConnectivityManager"), args, instantiate)



from kvdroid import activity
from kvdroid.jclass.android import TelephonySms
from datetime import datetime

get_date = datetime.fromtimestamp


def get_all_sms(content_resolver=activity.getContentResolver()):
    # sourcery skip: use-named-expression
    Sms = TelephonySms()
    cursor = content_resolver.query(Sms.CONTENT_URI, None, None, None, None)
    if cursor:
        total_sms = cursor.getCount()
        messages = []
        sms_types = {0: "all", 1: "inbox", 2: "sent", 3: "draft", 4: "outbox", 5: "failed", 6: "queued"}
        try:
            while cursor.moveToNext():
                messages.append({
                    "date": get_date(float(cursor.getString(cursor.getColumnIndexOrThrow("date"))) / 1000),
                    "number": cursor.getString(cursor.getColumnIndexOrThrow("address")),
                    "body": cursor.getString(cursor.getColumnIndexOrThrow("body")),
                    "type": sms_types[cursor.getInt(cursor.getColumnIndexOrThrow("type"))]
                })
        finally:
            cursor.close()
        return total_sms, messages
    return



from threading import Thread
from typing import Callable

from jnius import PythonJavaClass
from jnius.jnius import JavaException, java_method
from kvdroid.jclass.android import MediaPlayer, AudioManager
from kvdroid.cast import cast_object
from kvdroid import activity
from kvdroid.jclass.android import Context


class OnAudioFocusChangeListener(PythonJavaClass):
    __javainterfaces__ = ["android/media/AudioManager$OnAudioFocusChangeListener"]
    __javacontext__ = "app"

    def __init__(self, callback, **kwargs):
        super(OnAudioFocusChangeListener, self).__init__(**kwargs)
        self.callback = callback

    @java_method("(I)V")
    def onAudioFocusChange(self, focus_change):
        self.callback(focus_change)


class Player:
    def __init__(self, on_audio_focus_change: Callable = None):
        self.on_audio_focus_change = on_audio_focus_change
        self.mPlayer = None
        self.content = None

    def raw(self):
        return self.mPlayer

    def play(self, content: str):
        if not self.mPlayer:
            self.mPlayer = MediaPlayer(instantiate=True)
        self.content = content
        self.mPlayer.stop()
        self.mPlayer.reset()
        self.mPlayer.setDataSource(self.content)
        self.mPlayer.prepare()
        self.mPlayer.start()

    def pause(self):
        self.mPlayer.pause()

    def resume(self):
        self.mPlayer.start()

    def stop(self):
        self.mPlayer.stop()

    def stream(self, content: str, on_load_finish: Callable = lambda: None):
        Thread(target=self._stream, args=(content, on_load_finish)).start()

    def _stream(self, content: str, on_load_finish: Callable):
        if not self.mPlayer:
            self.mPlayer = MediaPlayer(instantiate=True)
        try:
            from kivy.clock import mainthread  # NOQA
            @mainthread  # NOQA
            def load_finish():
                on_load_finish()
        except ImportError:
            from android.runnable import run_on_ui_thread  # NOQA
            @run_on_ui_thread  # NOQA
            def load_finish():
                on_load_finish()
        self.content = content
        self.mPlayer.setAudioStreamType(AudioManager().STREAM_MUSIC)
        self.mPlayer.stop()
        self.mPlayer.reset()
        self.mPlayer.setDataSource(self.content)
        self.mPlayer.prepare()
        self.mPlayer.start()
        load_finish()

    def get_duration(self):
        if self.content:
            return self.mPlayer.getDuration()

    def current_position(self):
        if self.content:
            return self.mPlayer.getCurrentPosition()

    def seek(self, value: int):
        try:
            self.mPlayer.seekTo(value * 1000)
        except JavaException:
            pass

    def do_loop(self, loop=False):
        if not loop:
            self.mPlayer.setLooping(False)
        else:
            self.mPlayer.setLooping(True)

    def is_playing(self):
        return self.mPlayer.isPlaying()

    def release_media_player(self):
        if not self.mPlayer:
            return
        self.mPlayer.release()
        self.mPlayer = None

    def _on_audio_focus_change(self, focus_change):
        if focus_change == AudioManager().AUDIOFOCUS_LOSS:
            self.stop()
        elif focus_change == AudioManager().AUDIOFOCUS_LOSS_TRANSIENT:
            self.pause()
        elif focus_change == AudioManager().AUDIOFOCUS_LOSS_TRANSIENT_CAN_DUCK:
            self.mPlayer.setVolume(0.2, 0.2)
        elif focus_change == AudioManager().AUDIOFOCUS_GAIN:
            self.mPlayer.setVolume(1.0, 1.0)
            self.resume()

    def user_defined_focus_change(self, focus_change):
        if focus_change == AudioManager().AUDIOFOCUS_LOSS:
            self.on_audio_focus_change("AUDIOFOCUS_LOSS")
        elif focus_change == AudioManager().AUDIOFOCUS_LOSS_TRANSIENT:
            self.on_audio_focus_change("AUDIOFOCUS_LOSS_TRANSIENT")
        elif focus_change == AudioManager().AUDIOFOCUS_LOSS_TRANSIENT_CAN_DUCK:
            self.on_audio_focus_change("AUDIOFOCUS_LOSS_TRANSIENT_CAN_DUCK")
        elif focus_change == AudioManager().AUDIOFOCUS_GAIN:
            self.on_audio_focus_change("AUDIOFOCUS_GAIN")

    def request_audio_focus(self):
        focusChangeListener = OnAudioFocusChangeListener(
            callback=self.user_defined_focus_change or self._on_audio_focus_change)
        am = cast_object("audioManager", activity.getSystemService(Context().AUDIO_SERVICE))
        result = am.requestAudioFocus(focusChangeListener, AudioManager().STREAM_MUSIC, AudioManager().AUDIOFOCUS_GAIN)
        if result == AudioManager().AUDIOFOCUS_REQUEST_GRANTED:
            return True
        return False



from kvdroid.event import EventDispatcher
from kvdroid.base import EventLoop
from android.runnable import run_on_ui_thread
from kvdroid import activity
from kvdroid.jclass.android import RelativeLayout
from kvdroid.jclass.android import ViewGroupLayoutParams
from kvdroid import Logger


class App(EventDispatcher):
    # Return the current running App instance
    _running_app = None

    def __init__(self, **kwargs):
        App._running_app = self
        super().__init__(**kwargs)
        self.register_event_type("on_pause")
        self.register_event_type("on_create")
        self.register_event_type("on_destroy")
        self.register_event_type("on_resume")
        self._eventloop = EventLoop()

        #: The *root* widget returned by the :meth:`build_view`
        self.root = None

    def build_view(self):
        """Initializes the application; it will be called only once.
        If this method returns a widget (tree), it will be used as the root
        widget and added to the window.

        :return:
            None or a root :class:`~android.widget.RelativeLayout` instance
            if no self.root exists."""
        if not self.root:
            return RelativeLayout(activity)

    @run_on_ui_thread
    def add_content_view(self, layout):
        layout_params = ViewGroupLayoutParams()
        activity.addContentView(
            layout,
            layout_params(layout_params.MATCH_PARENT, layout_params.FILL_PARENT)
        )
        return layout

    def on_create(self):
        """Event handler for the `on_create` event which is fired after
        initialization (after build() has been called) but before the
        application has started running.
        """

    def on_pause(self):
        """Event handler called when Pause mode is requested"""

    def on_destroy(self):
        """Event handler for the `on_destroy` event which is fired when the
        application has finished running (i.e. the window is about to be
        closed).
        """

    def on_resume(self):
        pass

    def run(self):
        self.root = self.build_view()
        if not self.root:
            Logger.critical("Application: No Layout was returned in build")
            return
        self.add_content_view(self.root)
        self._eventloop.status = "created"
        self.dispatch("on_create")
        self._eventloop.mainloop()

    @staticmethod
    def get_running_app():
        """Return the currently running application instance.
        """
        return App._running_app

    def stop(self):
        self.dispatch("on_destroy")
        self._eventloop.close()
        App._running_app = None



import re

from kvdroid import activity
from kvdroid.jclass.android.graphics import Point


class Metrics(object):
    config = activity.getResources().getConfiguration()
    metric = activity.getResources().getDisplayMetrics()

    def height_dp(self):
        return self.config.screenHeightDp

    def width_dp(self):
        return self.config.screenWidthDp

    def height_px(self):
        return self.metric.heightPixels

    def width_px(self):
        return self.metric.widthPixels

    def orientation(self):
        if self.config.orientation == 1:
            return "portrait"
        else:
            return "landscape"

    @staticmethod
    def resolution():
        point = Point(instantiate=True)
        activity.getWindowManager().getDefaultDisplay().getRealSize(point)
        size = re.findall(r"\d+", point.toString())
        return size[1] + "x" + size[0]



from jnius import autoclass
from kvdroid.jclass import _class_call


def CameraDevice(*args, instantiate: bool = False):
    return _class_call(autoclass("android.hardware.camera2.CameraDevice"), args, instantiate)


def CameraCaptureSession(*args, instantiate: bool = False):
    return _class_call(autoclass("android.hardware.camera2.CameraCaptureSession"), args, instantiate)


def CaptureRequest(*args, instantiate: bool = False):
    return _class_call(autoclass("android.hardware.camera2.CaptureRequest"), args, instantiate)


def CaptureRequestBuilder(*args, instantiate: bool = False):
    return _class_call(autoclass("android.hardware.camera2.CaptureRequest$Builder"), args, instantiate)


def CameraManager(*args, instantiate: bool = False):
    return _class_call(autoclass("android.hardware.camera2.CameraManager"), args, instantiate)


def CameraCharacteristics(*args, instantiate: bool = False):
    return _class_call(autoclass("android.hardware.camera2.CameraCharacteristics"), args, instantiate)



from typing import Union

from kvdroid.cast import cast_object
from kvdroid.jclass.android import Bitmap, CompressFormat, Config, Canvas, AdaptiveIconDrawable, BitmapDrawable
from kvdroid.jclass.java import InputStream
from kvdroid.jclass.java import FileOutputStream
from kvdroid import activity
from kvdroid.jclass.android import BitmapFactory


def save_drawable(drawable, path, name):
    if isinstance(drawable, AdaptiveIconDrawable()):
        drawable = cast_object("adaptiveIconDrawable", drawable)
    else:
        drawable = cast_object("bitmapDrawable", drawable)

    height = drawable.getIntrinsicHeight() if drawable.getIntrinsicHeight() > 0 else 1
    width = drawable.getIntrinsicWidth() if drawable.getIntrinsicWidth() > 0 else 1
    if drawable.isFilterBitmap():
        bitmap = drawable.getBitmap()
    else:
        bitmap = Bitmap().createBitmap(width, height, Config().ARGB_8888)
        canvas = Canvas(bitmap)
        drawable.setBounds(0, 0, canvas.getWidth(), canvas.getHeight())
        drawable.draw(canvas)
    out = FileOutputStream(path + name + ".png")
    bitmap.compress(CompressFormat().PNG, 90, out)
    return path + name + ".png"


def bitmap_to_drawable(bitmap: Union[int, str, InputStream()]):
    BitmapFactory = BitmapFactory()
    if isinstance(bitmap, int):
        bitmap = BitmapFactory.decodeResource(activity.getResources(), bitmap)
    elif isinstance(bitmap, str):
        bitmap = BitmapFactory.decodeFile(bitmap)
    else:
        bitmap = BitmapFactory.decodeStream(bitmap)
    return BitmapDrawable(activity.getResource(), bitmap)



from jnius import autoclass
from kvdroid.jclass import _class_call


def File(*args, instantiate: bool = False):
    return _class_call(autoclass('java.io.File'), args, instantiate)


def FileOutputStream(*args, instantiate: bool = False):
    return _class_call(autoclass('java.io.FileOutputStream'), args, instantiate)


def FileInputStream(*args, instantiate: bool = False):
    return _class_call(autoclass('java.io.FileInputStream'), args, instantiate)


def ByteArrayOutputStream(*args, instantiate: bool = False):
    return _class_call(autoclass('java.io.ByteArrayOutputStream'), args, instantiate)


def InputStream(*args, instantiate: bool = False):
    return _class_call(autoclass('java.io.InputStream'), args, instantiate)



from jnius import autoclass
from kvdroid.jclass import _class_call


def PackageManager(*args, instantiate: bool = False):
    return _class_call(autoclass("android.content.pm.PackageManager"), args, instantiate)



"""
from kivy.app import App
from kivy.properties import StringProperty
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.screenmanager import Screen
from kivy.uix.recycleview import RecycleView
from kivy.lang import Builder
from kvdroid.jclass.java import Locale

from kvdroid.tools.font import system_font

Builder.load_string(""

<MainApp>:
    RCList:
        id: rclist
        size_hint: 1,1
    
<RCList>:
    viewclass: 'XBox'
    RecycleBoxLayout:
        spacing:  dp(10)
        #default_size: dp(100), dp(100)
        default_size_hint: 1, None
        size_hint_y: None
        height: self.minimum_height
        orientation: 'vertical'
<XBox>:
    Button:
        text: root.name
        font_size: "22sp"
    Button:
        text: root.native
        font_name: root.font_name
        font_size: "22sp"
        on_press: print(root.font_name)


"")


class XBox(BoxLayout):
    name = StringProperty()
    native = StringProperty()
    font_name = StringProperty()


class RCList(RecycleView):
    def __init__(self, **kwargs):
        super(RCList, self).__init__(**kwargs)
        self.data = []


class MainApp(Screen):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        for locale in Locale().getAvailableLocales():
            try:
                item = {"name": locale.getDisplayLanguage(Locale("en")),
                        "native": locale.getDisplayLanguage(Locale(locale.getLanguage())),
                        "font_name": system_font(locale.getLanguage())}
                if system_font(locale.getLanguage()) != "Roboto" and not item in self.ids.rclist.data:
                    self.ids.rclist.data.append(item)
            except:
                print(locale.getLanguage(), locale.getDisplayLanguage())


class Test(App):
    def build(self):
        return MainApp()


Test().run()
"""


import re
import os
from kvdroid.tools.iso import iso_codes
from kivy.core.text import LabelBase
from kvdroid.tools.lang import device_lang


def system_font(language=None):
    if language != None:
        if language.lower() in iso_codes.keys():
            try:
                return font_dict[iso_codes[language.lower()]]
            except:
                return "Roboto"
        else:
            raise ValueError(
                "The language definition must be in iso639-1 or iso639-2 code formats such as 'en' or 'eng'")
    else:
        locale = device_lang()
        try:
            return font_dict[iso_codes[locale]]
        except:
            return "Roboto"


def register_font(lang, font):
    if not lang in font_dict.keys():
        font_dict[lang] = font["name"]
        LabelBase.register(name=font["name"],
                           fn_regular=path + font["regular"],
                           fn_bold=path +
                           font["bold"] if "bold" in font.keys() else None,
                           fn_italic=path +
                           font["italic"] if "italic" in font.keys() else None,
                           fn_bolditalic=path + font["bolditalic"] if "bolditalic" in font.keys() else None)


def is_font_exist(font):
    if os.path.exists(path + font):
        return font
    else:
        new_font = font.split('.')[0]+'.otf'
        if os.path.exists(new_font):
            return new_font
        else:
            return None


def define_font(lang, item):
    temp = {}
    split_item = item.splitlines()
    for f in split_item:
        font = re.findall(
            "(?<=\s)[A-Z].*\.ttf|(?<=\s)[A-Z].*\.ttc|(?<=\s)[A-Z].*\.otf|(?<=\">)[A-Z].*\.ttf|(?<=\">)[A-Z].*\.ttc|(?<=\">)[A-Z].*\.otf", f)
        if font:
            font = is_font_exist(font[0])
            if font:
                name = font.split("-")[0]
                if not name.startswith('Roboto'):
                    temp["name"] = name
                    if "-Regular" in font:
                        if not "regular" in temp.keys():
                            temp["regular"] = font
                    elif "-Bold" in font:
                        if not "bold" in temp.keys():
                            temp["bold"] = font
                    elif "-Italic" in font:
                        if not "italic" in temp.keys():
                            temp["italic"] = font
                    elif "-BoldItalic" in font:
                        if not "bolditalic" in font:
                            temp["bolditalic"] = font
                    else:
                        if not "regular" in temp.keys():
                            temp["regular"] = font
    if temp:
        register_font(lang, temp)


path = "/system/fonts/"
font_dict = {}

if os.path.exists("/system/etc/fonts.xml"):
    r = open("/system/etc/fonts.xml").read()
elif os.path.exists("/system/etc/system_fonts.xml"):
    r = open("/system/etc/system_fonts.xml").read()
else:
    pass
if r:
    langs = re.findall("<family lang=[\s\S]*?</family>", r)
    if langs:
        for i in langs:
            lang = re.findall("\"(.*?)\"", i)[0]
            split_lang = lang.split(',')
            for s in split_lang:
                lang = s.split("-")[-1]
                if lang == "ja":
                    lang = "Jpan"
                if lang == "ko":
                    lang = "Kore"
                if lang == 'Hans':
                    define_font('Hant', i)
                define_font(lang, i)



from jnius import cast

# experimental(castables not yet complete, still in hunt)
castable_packages = {
    "parcelable": "android.os.Parcelable",
    "activity": "android.app.Activity",
    "context": "android.content.Context",
    "downloadManager": "android.app.DownloadManager",
    "charSequence": "java.lang.CharSequence",
    "bitmapDrawable": "android.graphics.drawable.BitmapDrawable",
    "adaptiveIconDrawable": "android.graphics.drawable.AdaptiveIconDrawable",
    "audioManager": "android.media.AudioManager",
    "uri": "android.net.Uri"
}


def cast_object(name: str, java_object: object):
    if name not in castable_packages:
        raise NameError("Java package name not in predefined castables")
    return cast(castable_packages[name], java_object)



from typing import List

from kvdroid.tools import toast
from kvdroid.cast import cast_object

from kvdroid import activity
from kvdroid.jclass.java import File, String
from kvdroid.jclass.android import Intent, VERSION
from kvdroid.jclass.androidx import FileProvider
from kvdroid.tools.uri import grant_uri_permission

context = activity.getApplicationContext()


def send_email(recipient: List[str], subject: str, body: str, file_path: str = None,
               create_chooser: bool = False, mime_type: str = "message/rfc822",
               authority: str = f"{context.getPackageName()}.fileprovider") -> None:
    """
    This method will open any default or selected email app and sends a body of message
    with or without an attachment depending on the chosen argument

    :param authority: file provider authority name
    :param recipient: email address of the receiver
    :param subject: subject of the email
    :param body: body of the email
    :param file_path: file path of the attachment to be sent (default is None)
    :param create_chooser: whether to create an option to choose a specific app for sending email
    :param mime_type: body text type (defaults to "text/plain")
    :rtype: None
    """
    if VERSION().SDK_INT >= 24:
        from kvdroid.jclass.android import StrictMode
        StrictMode().disableDeathOnFileUriExposure()
    uri = None
    email_intent = Intent(Intent().ACTION_SEND)
    email_intent.setType(mime_type)
    email_intent.putExtra(Intent().EXTRA_EMAIL, recipient)
    email_intent.putExtra(Intent().EXTRA_SUBJECT, String(subject))
    email_intent.putExtra(Intent().EXTRA_TEXT, String(body))
    email_intent.addFlags(Intent().FLAG_GRANT_READ_URI_PERMISSION | Intent().FLAG_GRANT_WRITE_URI_PERMISSION)
    if file_path:
        file = File(file_path)
        if not file.exists() or not file.canRead():
            toast(f"{file_path} does not exist")
            return
        uri = FileProvider().getUriForFile(activity, authority, file)
        parcelable = cast_object('parcelable', uri)
        email_intent.setDataAndType(uri, activity.getContentResolver().getType(uri))
        email_intent.putExtra(Intent().EXTRA_STREAM, parcelable)
    if create_chooser:
        chooser_intent = Intent().createChooser(
            email_intent, cast_object("charSequence", String("Pick an Email Provider")))
        if file_path:
            grant_uri_permission(chooser_intent, uri,
                                 Intent().FLAG_GRANT_WRITE_URI_PERMISSION |
                                 Intent().FLAG_GRANT_READ_URI_PERMISSION)
        activity.startActivity(chooser_intent)
    else:
        email_intent.setClassName('com.google.android.gm', 'com.google.android.gm.ComposeActivityGmailExternal')
        if file_path:
            context.grantUriPermission(String('com.google.android.gm'), uri,
                                       Intent().FLAG_GRANT_WRITE_URI_PERMISSION |
                                       Intent().FLAG_GRANT_READ_URI_PERMISSION)
        activity.startActivity(email_intent)



from jnius import autoclass
from kvdroid.jclass import _class_call


def Locale(*args, instantiate: bool = False):
    return _class_call(autoclass('java.util.Locale'), args, instantiate)


def ArrayList(*args, instantiate: bool = False):
    return _class_call(autoclass('java.util.ArrayList'), args, instantiate)



from kvdroid.event import EventDispatcher
from kvdroid import activity


class EventLoop(EventDispatcher):
    def __init__(self):
        super(EventLoop, self).__init__()
        from kvdroid.app import App
        self.app = App.get_running_app()
        self.quit = False
        self.status = "idle"
        self.resumed = False
        self.destroyed = False
        self.paused = False

    def mainloop(self):
        while not self.quit and self.status == "created":
            self.poll()

    def poll(self):
        if activity.isResumed() and not self.resumed:
            self.app.dispatch("on_resume")
            self.resumed = activity.resumed
            self.paused = False

        if activity.isDestroyed() and not self.destroyed:
            self.app.dispatch("on_destroy")
            self.destroyed = activity.destroyed
            self.resumed = False

        if not activity.hasWindowFocus() and not self.paused:
            self.app.dispatch("on_pause")
            self.paused = True
            self.resumed = False

    def close(self):
        self.quit = True
        self.status = "destroyed"





from kvdroid.jclass.android import Uri, Intent, CallLogCalls
from kvdroid import activity
from datetime import datetime

get_date = datetime.fromtimestamp


def make_call(tel):
    intent = Intent(Intent().ACTION_CALL, Uri().parse(f"tel:{tel}"))
    activity.startActivity(intent)


def dial_call(tel):
    intent = Intent(Intent().ACTION_DIAL, Uri().parse(f"tel:{tel}"))
    activity.startActivity(intent)


def get_call_log(content_resolver=activity.getContentResolver()):
    # sourcery skip: use-named-expression
    Calls = CallLogCalls()
    cursor = content_resolver.query(Calls.CONTENT_URI, None, None, None, None)
    if cursor:
        total_log = cursor.getCount()
        call_logs = []
        call_logs_type = {1: "incoming", 2: "outgoing", 3: "missed", 4: "voicemail", 5: "rejected", 6: "blocked"}
        try:
            while cursor.moveToNext():
                call_logs.append({
                    "date": get_date(float(cursor.getString(cursor.getColumnIndexOrThrow(Calls.DATE))) / 1000),
                    "name": cursor.getString(cursor.getColumnIndexOrThrow(Calls.CACHED_NAME)),
                    "number": cursor.getString(cursor.getColumnIndexOrThrow(Calls.NUMBER)),
                    "duration": cursor.getString(cursor.getColumnIndexOrThrow(Calls.DURATION)),
                    "type": call_logs_type[cursor.getInt(cursor.getColumnIndexOrThrow(Calls.TYPE))]
                })
        finally:
            cursor.close()
        return total_log, call_logs
    return



from jnius import autoclass
from kvdroid.jclass import _class_call


def URLConnection(*args, instantiate: bool = False):
    return _class_call(autoclass("java.net.URLConnection"), args, instantiate)


def HttpURLConnection(*args, instantiate: bool = False):
    return _class_call(autoclass("java.net.HttpURLConnection"), args, instantiate)


def URL(*args, instantiate: bool = False):
    return _class_call(autoclass("java.net.URL"), args, instantiate)



from jnius import autoclass
from kvdroid.jclass import _class_call


def ActivityCompat(*args, instantiate: bool = False):
    return _class_call(autoclass("android.support.v4.app.ActivityCompat"), args, instantiate)



from kvdroid import activity


def get_contact_details(option: str = "phone_book"):
    from kvdroid.jclass.android import Phone
    Phone = Phone()
    """
    option accepts this values : "phone_book", "mobile_no", "names"

    :param option: str: used to determine the return value
    :return: value
    """
    value = None
    PROJECTION = ["contact_id", "display_name", Phone.NUMBER]
    cr = activity.getContentResolver()
    cursor = cr.query(Phone.CONTENT_URI, PROJECTION, None, None, "display_name" + " ASC")
    mobile_no_set: list = []
    phone_book: dict = {}
    if cursor:
        try:
            name_index: int = cursor.getColumnIndex("display_name")
            number_index: int = cursor.getColumnIndex(Phone.NUMBER)

            while cursor.moveToNext():
                name = cursor.getString(name_index)
                number = cursor.getString(number_index)
                number = number.replace(" ", "")
                if number not in mobile_no_set:
                    if name in phone_book:
                        phone_book[name].append(number)
                    else:
                        phone_book[name] = [number]
                    mobile_no_set.append(number)
        finally:
            cursor.close()

        if option == "mobile_no":
            value = mobile_no_set
        elif option == "names":
            value = list(phone_book.keys())
        elif option == "phone_book":
            value = phone_book
        else:
            raise TypeError("available options are ['names', 'mobile_no', 'phone_book'] for get_contact_details")
    return value



from kvdroid.jclass.java.util import Locale


def device_lang(option="Language", display_lang=None):
    locale = Locale().getDefault()
    options = {
        "Language": locale.getLanguage(),
        "ISO3Language": locale.getISO3Language(),
        "Country": locale.getCountry(),
        "ISO3Country": locale.getISO3Country(),
        "DisplayCountry": locale.getDisplayCountry(Locale(str(display_lang))),
        "DisplayName": locale.getDisplayName(Locale(str(display_lang))),
        "String": locale.toString(),
        "DisplayLanguage": locale.getDisplayLanguage(Locale(str(display_lang))),
        "LanguageTag": locale.toLanguageTag()}

    if option not in options.keys():
        raise ValueError(f"Invalid option. Expected one of: {list(options.keys())}")
    else:
        return options[option]


def supported_languages():
    langs = []
    for locale in Locale().getAvailableLocales():
        if locale.getLanguage() not in langs:
            langs.append(locale.getLanguage())
    return langs



from jnius import autoclass
from kvdroid.jclass import _class_call


def ImageReader(*args, instantiate: bool = False):
    return _class_call(autoclass("android.media.ImageReader"), args, instantiate)


def MediaPlayer(instantiate: bool=False, *args):
    return _class_call(autoclass('android.media.MediaPlayer'), args, instantiate)


def AudioManager(instantiate: bool=False, *args):
    return _class_call(autoclass('android.media.AudioManager'), args, instantiate)




from jnius import autoclass
from kvdroid.jclass import _class_call


def Toast(*args, instantiate: bool = False):
    return _class_call(autoclass('android.widget.Toast'), args, instantiate)


def RelativeLayout(*args, instantiate: bool = False):
    return _class_call(autoclass('android.widget.RelativeLayout'), args, instantiate)


def LinearLayout(*args, instantiate: bool = False):
    return _class_call(autoclass('android.widget.LinearLayout'), args, instantiate)


def TextView(*args, instantiate: bool = False):
    return _class_call(autoclass('android.widget.TextView'), args, instantiate)



from jnius import autoclass
from kvdroid.jclass import _class_call


def Runtime(*args, instantiate: bool = False):
    return _class_call(autoclass('java.lang.Runtime'), args, instantiate)


def String(*args, instantiate: bool = False):
    return _class_call(autoclass("java.lang.String"), args, instantiate)


def System(*args, instantiate: bool = False):
    return _class_call(autoclass("java.lang.System"), args, instantiate)



from jnius import autoclass
from kvdroid.jclass import _class_call


def Environment(*args, instantiate: bool = False):
    return _class_call(autoclass("android.os.Environment"), args, instantiate)


def Build(*args, instantiate: bool = False):
    return _class_call(autoclass("android.os.Build"), args, instantiate)


def VERSION(*args, instantiate: bool = False):
    return _class_call(autoclass('android.os.Build$VERSION'), args, instantiate)


def VERSION_CODES(*args, instantiate: bool = False):
    return _class_call(autoclass("android.os.Build$VERSION_CODES"), args, instantiate)


def StrictMode(*args, instantiate: bool = False):
    return _class_call(autoclass('android.os.StrictMode'), args, instantiate)


def StatFs(*args, instantiate: bool = False):
    return _class_call(autoclass("android.os.StatFs"), args, instantiate)


def BatteryManager(*args, instantiate: bool = False):
    return _class_call(autoclass("android.os.BatteryManager"), args, instantiate)



from kvdroid.jclass.android import Activity
from kvdroid import activity
from jnius import JavaException, cast
from kvdroid.jclass.android import WifiManager, Formatter, Context


def network_status() -> bool:
    """
    Checks if Mobile data or Wi-Fi network is connected

    :rtype: bool
    :return: network status
    """
    return bool(wifi_status() or mobile_status())


def wifi_status() -> bool:
    """
    Checks if th phone Wi-Fi is connected to any network

    :rtype: bool
    :return: Wi-Fi status
    """
    from kvdroid.jclass.android import ConnectivityManager
    ConnectivityManager = ConnectivityManager()
    con_mgr = activity.getSystemService(Activity().CONNECTIVITY_SERVICE)
    try:
        return con_mgr.getNetworkInfo(ConnectivityManager.TYPE_WIFI).isConnectedOrConnecting()
    except JavaException:
        return False


def mobile_status() -> bool:
    """
    Checks if Mobile data is connected

    :rtype: bool
    :return: Mobile data status
    """
    from kvdroid.jclass.android import ConnectivityManager
    ConnectivityManager = ConnectivityManager()
    con_mgr = activity.getSystemService(Activity().CONNECTIVITY_SERVICE)
    try:
        return con_mgr.getNetworkInfo(ConnectivityManager.TYPE_MOBILE).isConnectedOrConnecting()
    except JavaException:
        return False


def get_wifi_ip_address() -> str:
    """
    Gets the Wi-Fi IP Address. But when the Wi-Fi is off, it
    returns a string value of "0.0.0.0"

    :rtype: str
    :return: Wi-Fi Ip Address
    """
    formatter = Formatter()
    context = activity.getApplicationContext()
    wifi_manager = cast(WifiManager(), context.getSystemService(Context().WIFI_SERVICE))
    return formatter.formatIpAddress(wifi_manager.getConnectionInfo().getIpAddress())



iso_codes = {
 'aa': 'Latn',
 'aar': 'Latn',
 'ab': 'Cyrl',
 'abk': 'Cyrl',
 'abq': 'Cyrl',
 'abr': 'Latn',
 'ace': 'Latn',
 'ach': 'Latn',
 'ada': 'Latn',
 'ady': 'Cyrl',
 'aeb': 'Arab',
 'af': 'Latn',
 'afr': 'Latn',
 'agq': 'Latn',
 'aii': 'Syrc',
 'ak': 'Latn',
 'aka': 'Latn',
 'akz': 'Latn',
 'ale': 'Latn',
 'aln': 'Latn',
 'alt': 'Cyrl',
 'am': 'Ethi',
 'amh': 'Ethi',
 'amo': 'Latn',
 'an': 'Latn',
 'anp': 'Deva',
 'aoz': 'Latn',
 'apu': 'Latn',
 'ar': 'Arab',
 'ara': 'Arab',
 'arg': 'Latn',
 'arn': 'Latn',
 'aro': 'Latn',
 'arp': 'Latn',
 'arq': 'Arab',
 'ars': 'Arab',
 'ary': 'Arab',
 'arz': 'Arab',
 'as': 'Beng',
 'asa': 'Latn',
 'asm': 'Beng',
 'ast': 'Latn',
 'atj': 'Latn',
 'av': 'Cyrl',
 'ava': 'Cyrl',
 'awa': 'Deva',
 'ay': 'Latn',
 'aym': 'Latn',
 'az': 'Latn',
 'aze': 'Latn',
 'ba': 'Cyrl',
 'bak': 'Cyrl',
 'bal': 'Arab',
 'bam': 'Latn',
 'ban': 'Latn',
 'bap': 'Deva',
 'bar': 'Latn',
 'bas': 'Latn',
 'bax': 'Bamu',
 'bbc': 'Latn',
 'bbj': 'Latn',
 'bci': 'Latn',
 'be': 'Cyrl',
 'bej': 'Arab',
 'bel': 'Cyrl',
 'bem': 'Latn',
 'ben': 'Beng',
 'bew': 'Latn',
 'bez': 'Latn',
 'bfd': 'Latn',
 'bfq': 'Taml',
 'bft': 'Arab',
 'bfy': 'Deva',
 'bg': 'Cyrl',
 'bgc': 'Deva',
 'bgn': 'Arab',
 'bgx': 'Grek',
 'bhb': 'Deva',
 'bhi': 'Deva',
 'bho': 'Deva',
 'bi': 'Latn',
 'bik': 'Latn',
 'bin': 'Latn',
 'bis': 'Latn',
 'bjj': 'Deva',
 'bjn': 'Latn',
 'bkm': 'Latn',
 'bku': 'Latn',
 'bla': 'Cans',
 'blt': 'Tavt',
 'bm': 'Latn',
 'bmq': 'Latn',
 'bn': 'Beng',
 'bo': 'Tibt',
 'bod': 'Tibt',
 'bos': 'Latn',
 'bpy': 'Beng',
 'bqi': 'Arab',
 'bqv': 'Latn',
 'br': 'Latn',
 'bra': 'Deva',
 'bre': 'Latn',
 'brh': 'Arab',
 'brx': 'Deva',
 'bs': 'Latn',
 'bss': 'Latn',
 'bto': 'Latn',
 'btv': 'Deva',
 'bua': 'Cyrl',
 'buc': 'Latn',
 'bug': 'Latn',
 'bul': 'Cyrl',
 'bum': 'Latn',
 'bvb': 'Latn',
 'bxr': 'Cyrl',
 'byn': 'Ethi',
 'byv': 'Latn',
 'bze': 'Latn',
 'bzx': 'Latn',
 'ca': 'Latn',
 'cad': 'Latn',
 'car': 'Latn',
 'cat': 'Latn',
 'cay': 'Latn',
 'cch': 'Latn',
 'ccp': 'Cakm',
 'ce': 'Cyrl',
 'ceb': 'Latn',
 'ces': 'Latn',
 'cgg': 'Latn',
 'ch': 'Latn',
 'cha': 'Latn',
 'che': 'Cyrl',
 'chk': 'Latn',
 'chm': 'Cyrl',
 'cho': 'Latn',
 'chp': 'Latn',
 'chr': 'Cher',
 'chv': 'Cyrl',
 'chy': 'Latn',
 'cic': 'Latn',
 'ciw': 'Latn',
 'cja': 'Arab',
 'cjm': 'Cham',
 'cjs': 'Cyrl',
 'ckb': 'Arab',
 'ckt': 'Cyrl',
 'co': 'Latn',
 'cor': 'Latn',
 'cos': 'Latn',
 'cps': 'Latn',
 'cr': 'Latn',
 'cre': 'Latn',
 'crh': 'Cyrl',
 'crj': 'Latn',
 'crk': 'Latn',
 'crl': 'Latn',
 'crm': 'Cans',
 'crs': 'Latn',
 'cs': 'Latn',
 'csw': 'Cans',
 'ctd': 'Latn',
 'cv': 'Cyrl',
 'cy': 'Latn',
 'cym': 'Latn',
 'da': 'Latn',
 'dak': 'Latn',
 'dan': 'Latn',
 'dar': 'Cyrl',
 'dav': 'Latn',
 'dcc': 'Arab',
 'de': 'Latn',
 'del': 'Latn',
 'den': 'Latn',
 'deu': 'Latn',
 'dgr': 'Latn',
 'din': 'Latn',
 'div': 'Thaa',
 'dje': 'Latn',
 'dng': 'Cyrl',
 'dnj': 'Latn',
 'doi': 'Arab',
 'dsb': 'Latn',
 'dtm': 'Latn',
 'dtp': 'Latn',
 'dty': 'Deva',
 'dua': 'Latn',
 'dv': 'Thaa',
 'dyo': 'Latn',
 'dyu': 'Latn',
 'dz': 'Tibt',
 'dzo': 'Tibt',
 'ebu': 'Latn',
 'ee': 'Latn',
 'efi': 'Latn',
 'egl': 'Latn',
 'eka': 'Latn',
 'eky': 'Kali',
 'el': 'Grek',
 'ell': 'Grek',
 'en': 'Latn',
 'eng': 'Latn',
 'eo': 'Latn',
 'epo': 'Latn',
 'es': 'Latn',
 'ess': 'Latn',
 'est': 'Latn',
 'esu': 'Latn',
 'et': 'Latn',
 'eu': 'Latn',
 'eus': 'Latn',
 'evn': 'Cyrl',
 'ewe': 'Latn',
 'ewo': 'Latn',
 'ext': 'Latn',
 'fa': 'Arab',
 'fan': 'Latn',
 'fao': 'Latn',
 'fas': 'Arab',
 'ff': 'Latn',
 'ffm': 'Latn',
 'fi': 'Latn',
 'fia': 'Arab',
 'fij': 'Latn',
 'fil': 'Latn',
 'fin': 'Latn',
 'fit': 'Latn',
 'fj': 'Latn',
 'fkv': 'Latn',
 'fo': 'Latn',
 'fon': 'Latn',
 'fr': 'Latn',
 'fra': 'Latn',
 'frc': 'Latn',
 'frp': 'Latn',
 'frr': 'Latn',
 'frs': 'Latn',
 'fry': 'Latn',
 'fud': 'Latn',
 'ful': 'Latn',
 'fuq': 'Latn',
 'fur': 'Latn',
 'fuv': 'Latn',
 'fvr': 'Latn',
 'fy': 'Latn',
 'ga': 'Latn',
 'gaa': 'Latn',
 'gag': 'Latn',
 'gan': 'Hans',
 'gay': 'Latn',
 'gba': 'Latn',
 'gbm': 'Deva',
 'gbz': 'Arab',
 'gcr': 'Latn',
 'gd': 'Latn',
 'gil': 'Latn',
 'gjk': 'Arab',
 'gju': 'Arab',
 'gl': 'Latn',
 'gla': 'Latn',
 'gld': 'Cyrl',
 'gle': 'Latn',
 'glg': 'Latn',
 'glk': 'Arab',
 'glv': 'Latn',
 'gn': 'Latn',
 'gom': 'Deva',
 'gon': 'Telu',
 'gor': 'Latn',
 'gos': 'Latn',
 'grb': 'Latn',
 'grn': 'Latn',
 'grt': 'Beng',
 'gsw': 'Latn',
 'gu': 'Gujr',
 'gub': 'Latn',
 'guc': 'Latn',
 'guj': 'Gujr',
 'gur': 'Latn',
 'guz': 'Latn',
 'gv': 'Latn',
 'gvr': 'Deva',
 'gwi': 'Latn',
 'ha': 'Latn',
 'hai': 'Latn',
 'hak': 'Hans',
 'hat': 'Latn',
 'hau': 'Latn',
 'haw': 'Latn',
 'haz': 'Arab',
 'hdn': 'Latn',
 'he': 'Hebr',
 'heb': 'Hebr',
 'her': 'Latn',
 'hi': 'Deva',
 'hif': 'Latn',
 'hil': 'Latn',
 'hin': 'Deva',
 'hmd': 'Plrd',
 'hmn': 'Latn',
 'hmo': 'Latn',
 'hnd': 'Arab',
 'hne': 'Deva',
 'hnj': 'Laoo',
 'hnn': 'Latn',
 'hno': 'Arab',
 'ho': 'Latn',
 'hoc': 'Deva',
 'hoj': 'Deva',
 'hop': 'Latn',
 'hr': 'Latn',
 'hrv': 'Latn',
 'hsb': 'Latn',
 'hsn': 'Hans',
 'ht': 'Latn',
 'hu': 'Latn',
 'hun': 'Latn',
 'hup': 'Latn',
 'hy': 'Armn',
 'hye': 'Armn',
 'hz': 'Latn',
 'iba': 'Latn',
 'ibb': 'Latn',
 'ibo': 'Latn',
 'id': 'Latn',
 'ife': 'Latn',
 'ig': 'Latn',
 'ii': 'Yiii',
 'iii': 'Yiii',
 'ik': 'Latn',
 'ikt': 'Latn',
 'iku': 'Latn',
 'ilo': 'Latn',
 'ind': 'Latn',
 'inh': 'Cyrl',
 'ipk': 'Latn',
 'is': 'Latn',
 'isl': 'Latn',
 'it': 'Latn',
 'ita': 'Latn',
 'iu': 'Latn',
 'izh': 'Latn',
 'ja': 'Jpan',
 'jam': 'Latn',
 'jav': 'Latn',
 'jgo': 'Latn',
 'jmc': 'Latn',
 'jml': 'Deva',
 'jpn': 'Jpan',
 'jpr': 'Hebr',
 'jrb': 'Hebr',
 'jv': 'Latn',
 'ka': 'Geor',
 'kaa': 'Cyrl',
 'kab': 'Latn',
 'kac': 'Latn',
 'kaj': 'Latn',
 'kal': 'Latn',
 'kam': 'Latn',
 'kan': 'Knda',
 'kao': 'Latn',
 'kas': 'Deva',
 'kat': 'Geor',
 'kau': 'Latn',
 'kaz': 'Cyrl',
 'kbd': 'Cyrl',
 'kca': 'Cyrl',
 'kcg': 'Latn',
 'kck': 'Latn',
 'kde': 'Latn',
 'kdt': 'Thai',
 'kea': 'Latn',
 'kek': 'Latn',
 'kfo': 'Latn',
 'kfr': 'Deva',
 'kfy': 'Deva',
 'kg': 'Latn',
 'kge': 'Latn',
 'kgp': 'Latn',
 'kha': 'Latn',
 'khb': 'Talu',
 'khk': 'Cyrl',
 'khm': 'Khmr',
 'khn': 'Deva',
 'khq': 'Latn',
 'kht': 'Mymr',
 'khw': 'Arab',
 'ki': 'Latn',
 'kik': 'Latn',
 'kin': 'Latn',
 'kio': 'Latn',
 'kir': 'Latn',
 'kiu': 'Latn',
 'kj': 'Latn',
 'kjg': 'Laoo',
 'kjh': 'Cyrl',
 'kk': 'Cyrl',
 'kkj': 'Latn',
 'kl': 'Latn',
 'kln': 'Latn',
 'km': 'Khmr',
 'kmb': 'Latn',
 'kn': 'Knda',
 'ko': 'Kore',
 'koi': 'Cyrl',
 'kok': 'Deva',
 'kom': 'Cyrl',
 'kon': 'Latn',
 'kor': 'Kore',
 'kos': 'Latn',
 'kpe': 'Latn',
 'kpv': 'Cyrl',
 'kpy': 'Cyrl',
 'kr': 'Latn',
 'krc': 'Cyrl',
 'kri': 'Latn',
 'krj': 'Latn',
 'krl': 'Latn',
 'kru': 'Deva',
 'ks': 'Deva',
 'ksb': 'Latn',
 'ksf': 'Latn',
 'ksh': 'Latn',
 'ku': 'Latn',
 'kua': 'Latn',
 'kum': 'Cyrl',
 'kur': 'Latn',
 'kut': 'Latn',
 'kv': 'Cyrl',
 'kvr': 'Latn',
 'kvx': 'Arab',
 'kw': 'Latn',
 'kxm': 'Thai',
 'kxp': 'Arab',
 'ky': 'Latn',
 'kyu': 'Kali',
 'la': 'Latn',
 'lad': 'Hebr',
 'lag': 'Latn',
 'lah': 'Arab',
 'laj': 'Latn',
 'lam': 'Latn',
 'lao': 'Laoo',
 'lat': 'Latn',
 'lav': 'Latn',
 'lb': 'Latn',
 'lbe': 'Cyrl',
 'lbw': 'Latn',
 'lcp': 'Thai',
 'lep': 'Lepc',
 'lez': 'Cyrl',
 'lg': 'Latn',
 'li': 'Latn',
 'lif': 'Limb',
 'lij': 'Latn',
 'lim': 'Latn',
 'lin': 'Latn',
 'lis': 'Lisu',
 'lit': 'Latn',
 'liv': 'Latn',
 'ljp': 'Latn',
 'lki': 'Arab',
 'lkt': 'Latn',
 'lmn': 'Telu',
 'lmo': 'Latn',
 'ln': 'Latn',
 'lo': 'Laoo',
 'lol': 'Latn',
 'loz': 'Latn',
 'lrc': 'Arab',
 'lt': 'Latn',
 'ltg': 'Latn',
 'ltz': 'Latn',
 'lu': 'Latn',
 'lua': 'Latn',
 'lub': 'Latn',
 'lug': 'Latn',
 'lun': 'Latn',
 'luo': 'Latn',
 'lus': 'Beng',
 'lut': 'Latn',
 'luy': 'Latn',
 'luz': 'Arab',
 'lv': 'Latn',
 'lwl': 'Thai',
 'lzz': 'Latn',
 'mad': 'Latn',
 'maf': 'Latn',
 'mag': 'Deva',
 'mah': 'Latn',
 'mai': 'Deva',
 'mak': 'Latn',
 'mal': 'Mlym',
 'man': 'Nkoo',
 'mar': 'Deva',
 'mas': 'Latn',
 'maz': 'Latn',
 'mdf': 'Cyrl',
 'mdh': 'Latn',
 'mdr': 'Latn',
 'mdt': 'Latn',
 'men': 'Latn',
 'mer': 'Latn',
 'mfa': 'Arab',
 'mfe': 'Latn',
 'mg': 'Latn',
 'mgh': 'Latn',
 'mgo': 'Latn',
 'mgp': 'Deva',
 'mgy': 'Latn',
 'mh': 'Latn',
 'mhr': 'Cyrl',
 'mi': 'Latn',
 'mic': 'Latn',
 'min': 'Latn',
 'mk': 'Cyrl',
 'mkd': 'Cyrl',
 'ml': 'Mlym',
 'mlg': 'Latn',
 'mls': 'Latn',
 'mlt': 'Latn',
 'mn': 'Cyrl',
 'mni': 'Beng',
 'mns': 'Cyrl',
 'mnw': 'Mymr',
 'moe': 'Latn',
 'moh': 'Latn',
 'mon': 'Cyrl',
 'mos': 'Latn',
 'mr': 'Deva',
 'mrd': 'Deva',
 'mri': 'Latn',
 'mrj': 'Cyrl',
 'mro': 'Latn',
 'ms': 'Latn',
 'msa': 'Latn',
 'mt': 'Latn',
 'mtr': 'Deva',
 'mua': 'Latn',
 'mus': 'Latn',
 'mvy': 'Arab',
 'mwk': 'Latn',
 'mwl': 'Latn',
 'mwr': 'Deva',
 'mwv': 'Latn',
 'mxc': 'Latn',
 'my': 'Mymr',
 'mya': 'Mymr',
 'myv': 'Cyrl',
 'myx': 'Latn',
 'mzn': 'Arab',
 'na': 'Latn',
 'nan': 'Hans',
 'nap': 'Latn',
 'naq': 'Latn',
 'nau': 'Latn',
 'nav': 'Latn',
 'nb': 'Latn',
 'nbl': 'Latn',
 'nch': 'Latn',
 'nd': 'Latn',
 'ndc': 'Latn',
 'nde': 'Latn',
 'ndl': 'Latn',
 'ndo': 'Latn',
 'nds': 'Latn',
 'ne': 'Deva',
 'nep': 'Deva',
 'new': 'Deva',
 'ng': 'Latn',
 'ngl': 'Latn',
 'nhe': 'Latn',
 'nhw': 'Latn',
 'nia': 'Latn',
 'nij': 'Latn',
 'nio': 'Cyrl',
 'niu': 'Latn',
 'njo': 'Latn',
 'nl': 'Latn',
 'nld': 'Latn',
 'nmg': 'Latn',
 'nn': 'Latn',
 'nnh': 'Latn',
 'nno': 'Latn',
 'no': 'Latn',
 'nob': 'Latn',
 'nod': 'Lana',
 'noe': 'Deva',
 'nog': 'Cyrl',
 'non': 'Runr',
 'nor': 'Latn',
 'nqo': 'Nkoo',
 'nr': 'Latn',
 'nsk': 'Cans',
 'nso': 'Latn',
 'nus': 'Latn',
 'nv': 'Latn',
 'nxq': 'Latn',
 'ny': 'Latn',
 'nya': 'Latn',
 'nym': 'Latn',
 'nyn': 'Latn',
 'nyo': 'Latn',
 'nzi': 'Latn',
 'oc': 'Latn',
 'oci': 'Latn',
 'oj': 'Latn',
 'oji': 'Latn',
 'olo': 'Latn',
 'om': 'Latn',
 'or': 'Orya',
 'ori': 'Orya',
 'orm': 'Latn',
 'os': 'Cyrl',
 'osa': 'Osge',
 'oss': 'Cyrl',
 'pa': 'Guru',
 'pag': 'Latn',
 'pam': 'Latn',
 'pan': 'Guru',
 'pap': 'Latn',
 'pau': 'Latn',
 'pcd': 'Latn',
 'pcm': 'Latn',
 'pdc': 'Latn',
 'pdt': 'Latn',
 'pfl': 'Latn',
 'pko': 'Latn',
 'pl': 'Latn',
 'pms': 'Latn',
 'pnt': 'Latn',
 'pol': 'Latn',
 'pon': 'Latn',
 'por': 'Latn',
 'prd': 'Arab',
 'prs': 'Arab',
 'ps': 'Arab',
 'pt': 'Latn',
 'pus': 'Arab',
 'puu': 'Latn',
 'qu': 'Latn',
 'quc': 'Latn',
 'que': 'Latn',
 'qug': 'Latn',
 'quz': 'Latn',
 'raj': 'Deva',
 'rap': 'Latn',
 'rar': 'Latn',
 'rcf': 'Latn',
 'rej': 'Latn',
 'rgn': 'Latn',
 'ria': 'Latn',
 'rif': 'Tfng',
 'rjs': 'Deva',
 'rkt': 'Beng',
 'rm': 'Latn',
 'rmf': 'Latn',
 'rmn': 'Latn',
 'rmo': 'Latn',
 'rmt': 'Arab',
 'rmu': 'Latn',
 'rmy': 'Latn',
 'rn': 'Latn',
 'rng': 'Latn',
 'ro': 'Latn',
 'rob': 'Latn',
 'rof': 'Latn',
 'roh': 'Latn',
 'rom': 'Latn',
 'ron': 'Latn',
 'rtm': 'Latn',
 'ru': 'Cyrl',
 'rue': 'Cyrl',
 'rug': 'Latn',
 'run': 'Latn',
 'rup': 'Latn',
 'rus': 'Cyrl',
 'rw': 'Latn',
 'rwk': 'Latn',
 'ryu': 'Kana',
 'sa': 'Deva',
 'sad': 'Latn',
 'saf': 'Latn',
 'sag': 'Latn',
 'sah': 'Cyrl',
 'san': 'Deva',
 'saq': 'Latn',
 'sas': 'Latn',
 'sat': 'Olck',
 'saz': 'Saur',
 'sbp': 'Latn',
 'sc': 'Latn',
 'sck': 'Deva',
 'scn': 'Latn',
 'sco': 'Latn',
 'scs': 'Latn',
 'sd': 'Deva',
 'sdc': 'Latn',
 'sdh': 'Arab',
 'se': 'Latn',
 'see': 'Latn',
 'sef': 'Latn',
 'seh': 'Latn',
 'sei': 'Latn',
 'sel': 'Cyrl',
 'ses': 'Latn',
 'sg': 'Latn',
 'sgs': 'Latn',
 'shi': 'Tfng',
 'shn': 'Mymr',
 'si': 'Sinh',
 'sid': 'Latn',
 'sin': 'Sinh',
 'sjd': 'Cyrl',
 'sje': 'Latn',
 'sjt': 'Cyrl',
 'sk': 'Latn',
 'skr': 'Arab',
 'sl': 'Latn',
 'sli': 'Latn',
 'slk': 'Latn',
 'slv': 'Latn',
 'sly': 'Latn',
 'sm': 'Latn',
 'sma': 'Latn',
 'sme': 'Latn',
 'smj': 'Latn',
 'smn': 'Latn',
 'smo': 'Latn',
 'sms': 'Latn',
 'sn': 'Latn',
 'sna': 'Latn',
 'snd': 'Deva',
 'snk': 'Latn',
 'so': 'Latn',
 'som': 'Latn',
 'sot': 'Latn',
 'sou': 'Thai',
 'spa': 'Latn',
 'sq': 'Latn',
 'sqi': 'Latn',
 'sr': 'Latn',
 'srb': 'Latn',
 'srd': 'Latn',
 'srn': 'Latn',
 'srp': 'Latn',
 'srr': 'Latn',
 'srs': 'Latn',
 'srx': 'Deva',
 'ss': 'Latn',
 'ssw': 'Latn',
 'ssy': 'Latn',
 'st': 'Latn',
 'sto': 'Latn',
 'stq': 'Latn',
 'su': 'Latn',
 'suk': 'Latn',
 'sun': 'Latn',
 'sus': 'Latn',
 'sv': 'Latn',
 'sw': 'Latn',
 'swa': 'Latn',
 'swb': 'Arab',
 'swe': 'Latn',
 'swg': 'Latn',
 'swv': 'Deva',
 'sxn': 'Latn',
 'syi': 'Latn',
 'syl': 'Beng',
 'syr': 'Syrc',
 'szl': 'Latn',
 'ta': 'Taml',
 'tab': 'Cyrl',
 'tah': 'Latn',
 'taj': 'Deva',
 'tam': 'Taml',
 'tat': 'Cyrl',
 'tau': 'Latn',
 'tbw': 'Latn',
 'tcy': 'Knda',
 'tdd': 'Tale',
 'tdg': 'Deva',
 'tdh': 'Deva',
 'te': 'Telu',
 'tel': 'Telu',
 'tem': 'Latn',
 'teo': 'Latn',
 'ter': 'Latn',
 'tet': 'Latn',
 'tg': 'Latn',
 'tgk': 'Latn',
 'tgl': 'Latn',
 'th': 'Thai',
 'tha': 'Thai',
 'thl': 'Deva',
 'thq': 'Deva',
 'thr': 'Deva',
 'ti': 'Ethi',
 'tig': 'Ethi',
 'tir': 'Ethi',
 'tiv': 'Latn',
 'tk': 'Latn',
 'tkl': 'Latn',
 'tkr': 'Latn',
 'tkt': 'Deva',
 'tku': 'Latn',
 'tl': 'Latn',
 'tlh': 'Piqd',
 'tli': 'Latn',
 'tly': 'Latn',
 'tmh': 'Latn',
 'tn': 'Latn',
 'to': 'Latn',
 'tog': 'Latn',
 'ton': 'Latn',
 'tpi': 'Latn',
 'tr': 'Latn',
 'tru': 'Latn',
 'trv': 'Latn',
 'trw': 'Arab',
 'ts': 'Latn',
 'tsd': 'Grek',
 'tsg': 'Latn',
 'tsi': 'Latn',
 'tsj': 'Tibt',
 'tsn': 'Latn',
 'tso': 'Latn',
 'tt': 'Cyrl',
 'ttj': 'Latn',
 'tts': 'Thai',
 'ttt': 'Latn',
 'tuk': 'Latn',
 'tum': 'Latn',
 'tur': 'Latn',
 'tuv': 'Latn',
 'tvl': 'Latn',
 'twq': 'Latn',
 'ty': 'Latn',
 'tyv': 'Cyrl',
 'tzm': 'Tfng',
 'ude': 'Cyrl',
 'udm': 'Cyrl',
 'ug': 'Cyrl',
 'uig': 'Cyrl',
 'uk': 'Cyrl',
 'ukr': 'Cyrl',
 'uli': 'Latn',
 'umb': 'Latn',
 'unr': 'Deva',
 'unx': 'Deva',
 'ur': 'Arab',
 'urd': 'Arab',
 'uz': 'Latn',
 'uzb': 'Latn',
 'vai': 'Vaii',
 've': 'Latn',
 'vec': 'Latn',
 'ven': 'Latn',
 'vep': 'Latn',
 'vi': 'Latn',
 'vic': 'Latn',
 'vie': 'Latn',
 'vls': 'Latn',
 'vmf': 'Latn',
 'vmw': 'Latn',
 'vot': 'Latn',
 'vro': 'Latn',
 'vun': 'Latn',
 'wa': 'Latn',
 'wae': 'Latn',
 'wal': 'Ethi',
 'war': 'Latn',
 'was': 'Latn',
 'wbp': 'Latn',
 'wbq': 'Telu',
 'wbr': 'Deva',
 'wln': 'Latn',
 'wls': 'Latn',
 'wni': 'Arab',
 'wo': 'Latn',
 'wol': 'Latn',
 'wtm': 'Deva',
 'wuu': 'Hans',
 'xal': 'Cyrl',
 'xav': 'Latn',
 'xh': 'Latn',
 'xho': 'Latn',
 'xmf': 'Geor',
 'xnr': 'Deva',
 'xog': 'Latn',
 'xsr': 'Deva',
 'xwo': 'Mong',
 'yao': 'Latn',
 'yap': 'Latn',
 'yav': 'Latn',
 'ybb': 'Latn',
 'yi': 'Hebr',
 'yid': 'Hebr',
 'yo': 'Latn',
 'yor': 'Latn',
 'yrk': 'Cyrl',
 'yrl': 'Latn',
 'yua': 'Latn',
 'yue': 'Hant',
 'za': 'Latn',
 'zag': 'Latn',
 'zap': 'Latn',
 'zdj': 'Arab',
 'zea': 'Latn',
 'zgh': 'Tfng',
 'zh': 'Hant',
 'zha': 'Latn',
 'zho': 'Hant',
 'zmi': 'Latn',
 'zu': 'Latn',
 'zul': 'Latn',
 'zun': 'Latn',
 'zza': 'Latn'}



from kvdroid.jclass.android.app import MemoryInfo
from kvdroid.jclass.android import IntentFilter, Intent
from kvdroid.jclass.android import StatFs
from kvdroid.jclass.java import Runtime
from kvdroid import activity


def device_info(text, convert=False):
    from kvdroid.jclass.android import Context, Build, BatteryManager, VERSION, Environment
    Environment = Environment()
    VERSION = VERSION()
    Build = Build()
    BatteryManager = BatteryManager()
    Context = Context()
    bm = activity.getSystemService(Context.BATTERY_SERVICE)
    count = bm.getIntProperty(BatteryManager.BATTERY_PROPERTY_CHARGE_COUNTER)
    cap = bm.getIntProperty(BatteryManager.BATTERY_PROPERTY_CAPACITY)
    intent = activity.registerReceiver(None, IntentFilter(Intent().ACTION_BATTERY_CHANGED))

    def convert_bytes(num):
        step_unit = 1000.0  # 1024 bad the size
        for x in ['bytes', 'KB', 'MB', 'GB', 'TB']:
            if num < step_unit:
                return "%3.1f %s" % (num, x)
            num /= step_unit

    def avail_mem():
        stat = StatFs(Environment.getDataDirectory().getPath())
        bytesAvailable = stat.getBlockSize() * stat.getAvailableBlocks()
        if convert:
            return convert_bytes(bytesAvailable)
        else:
            return bytesAvailable

    def total_mem():
        stat = StatFs(Environment.getDataDirectory().getPath())
        bytesAvailable = stat.getBlockSize() * stat.getBlockCount()
        if convert:
            return convert_bytes(bytesAvailable)
        else:
            return bytesAvailable

    def used_mem():
        stat = StatFs(Environment.getDataDirectory().getPath())
        total = stat.getBlockSize() * stat.getBlockCount()
        avail = stat.getBlockSize() * stat.getAvailableBlocks()
        if convert:
            return convert_bytes(total - avail)
        else:
            return total - avail

    def avail_ram():
        memInfo = MemoryInfo(instantiate=True)
        service = activity.getSystemService(Context.ACTIVITY_SERVICE)
        service.getMemoryInfo(memInfo)
        if convert:
            return convert_bytes(memInfo.availMem)
        else:
            return memInfo.availMem

    def total_ram():
        memInfo = MemoryInfo(instantiate=True)
        service = activity.getSystemService(Context.ACTIVITY_SERVICE)
        service.getMemoryInfo(memInfo)
        if convert:
            return convert_bytes(memInfo.totalMem)
        else:
            return memInfo.totalMem

    def used_ram():
        memInfo = MemoryInfo(instantiate=True)
        service = activity.getSystemService(Context.ACTIVITY_SERVICE)
        service.getMemoryInfo(memInfo)
        if convert:
            return convert_bytes(memInfo.totalMem - memInfo.availMem)
        else:
            return memInfo.totalMem - memInfo.availMem

    os = {
        'model': Build.MODEL,
        'brand': Build.BRAND,
        'manufacturer': Build.MANUFACTURER,
        'version': VERSION.RELEASE,
        'sdk': VERSION.SDK,
        'product': Build.PRODUCT,
        'base': VERSION.BASE_OS,
        'rom': VERSION.INCREMENTAL,
        'security': VERSION.SECURITY_PATCH,
        'hardware': Build.HARDWARE,
        'tags': Build.TAGS,
        'sdk_int': VERSION.SDK_INT,
        'cpu_abi': Build.CPU_ABI,
        'cpu_cores': Runtime().getRuntime().availableProcessors(),
        'avail_mem': avail_mem(),
        'total_mem': total_mem(),
        'used_mem': used_mem(),
        'avail_ram': avail_ram(),
        'total_ram': total_ram(),
        'used_ram': used_ram(),
        'bat_level': bm.getIntProperty(BatteryManager.BATTERY_PROPERTY_CAPACITY),
        'bat_capacity': round((count / cap) * 100),
        'bat_tempeture': intent.getIntExtra(BatteryManager.EXTRA_TEMPERATURE, 0) / 10,
        'bat_voltage': float(intent.getIntExtra(BatteryManager.EXTRA_VOLTAGE, 0) * 0.001),
        'bat_technology': intent.getStringExtra(BatteryManager.EXTRA_TECHNOLOGY)
    }
    return os[text]



from kvdroid.jclass.android import PackageManager

from kvdroid import activity

context = activity.getApplicationContext()


def grant_uri_permission(intent, uri, permissions):
    packageManager = context.getPackageManager()
    activities = packageManager.queryIntentActivities(intent, PackageManager().MATCH_DEFAULT_ONLY)
    activities = activities.toArray()
    for resolveInfo in activities:
        packageName = resolveInfo.activityInfo.packageName
        context.grantUriPermission(packageName, uri, permissions)


def revoke_uri_permission(uri, permissions):
    context.revokeUriPermission(uri, permissions)



from typing import Union
from jnius import cast, JavaException  # NOQA
from android import python_act  # NOQA
from kvdroid import Logger, activity
from kvdroid.cast import cast_object
from kvdroid.jclass.android import BitmapFactory
from kvdroid.jclass.java import InputStream
from kvdroid.tools import get_resource
from kvdroid.tools.graphics import bitmap_to_drawable

BitmapFactory = BitmapFactory()

try:
    from kvdroid.jclass.androidx import (
        ContextCompat,
        NotificationCompatBuilder,
        NotificationCompat,
        NotificationCompatActionBuilder,
        RemoteInputBuilder,
        NotificationCompatBigPictureStyle,
        NotificationBigTextStyle
    )
    from kvdroid.jclass.android import (
        Intent,
        Context,
        PendingIntent,
        NotificationManager,
        Notification,
        VERSION
    )
    from kvdroid.jclass.java.lang import String, System


    def create_notification(
            small_icon: Union[int, str, InputStream()], channel_id: str, title: str,
            text: str, ids: int, channel_name: str,
            large_icon: Union[int, str, InputStream()] = 0, big_picture: Union[int, str, InputStream()] = 0,
            action_title1: str = "", action_title2: str = "", action_title3: str = "", key_text_reply: str = "",
            reply_title: str = "",
            expandable: bool = False, set_large_icon: bool = False, add_action_button2: bool = False,
            add_action_button3: bool = False, auto_cancel: bool = True, add_action_button1: bool = False,
            set_reply: bool = False, put_extra: bool = False, extras: tuple = ("test", "test"),
            small_icon_color: int = 0
    ):  # sourcery no-metrics
        intent = Intent(activity.getApplication().getApplicationContext(), python_act)
        if put_extra:
            intent.putExtra(*extras)
        intent.setFlags(Intent().FLAG_ACTIVITY_CLEAR_TOP | Intent().FLAG_ACTIVITY_SINGLE_TOP)
        intent.setAction(Intent().ACTION_MAIN)
        intent.addCategory(Intent().CATEGORY_LAUNCHER)
        if VERSION().SDK_INT >= 31:
            pendingIntent = PendingIntent().getActivity(
                activity.getApplication().getApplicationContext(), 0, intent,
                PendingIntent().FLAG_MUTABLE
            )
        else:
            pendingIntent = PendingIntent().getActivity(
                activity.getApplication().getApplicationContext(), 0, intent,
                PendingIntent().FLAG_UPDATE_CURRENT | Notification().FLAG_AUTO_CANCEL
            )
        # Create the NotificationChannel, but only on API 26+ because
        # the NotificationChannel class is new and not in the support library
        notificationManager = cast(NotificationManager(), activity.getSystemService(
            Context().NOTIFICATION_SERVICE
        ))
        if VERSION().SDK_INT >= 26:
            from kvdroid.jclass.android.app import NotificationChannel
            importance: int = NotificationManager().IMPORTANCE_HIGH
            channel = NotificationChannel(String(channel_id), String(channel_name), importance)
            channel.setDescription("notification")
            # Register the channel with the system; you can't change the importance
            # or other notification behaviors after this
            notificationManager.createNotificationChannel(channel)

        builder = NotificationCompatBuilder(activity.getApplication().getApplicationContext(), channel_id)
        if isinstance(small_icon, int):
            builder.setSmallIcon(small_icon)
        else:
            builder.setSmallIcon(bitmap_to_drawable(small_icon))
        if small_icon_color:
            try:
                builder.setColor(ContextCompat().getColor(activity.getApplicationContext(), small_icon_color))
            except JavaException:
                builder.setColor(small_icon_color)
        builder.setContentTitle(String(title))
        builder.setContentText(String(text))
        # Set the intent that will fire when the user taps the notification
        builder.setContentIntent(pendingIntent)
        builder.setWhen(System().currentTimeMillis())
        builder.setChannelId(String(channel_id))
        builder.setPriority(NotificationCompat().PRIORITY_HIGH)
        builder.setDefaults(NotificationCompat().DEFAULT_ALL)
        builder.setAutoCancel(auto_cancel)
        builder.setTicker(String(""))
        builder.setContentInfo("INFO")
        builder.setStyle(NotificationBigTextStyle(instantiate=True).bigText(String(text)).setBigContentTitle(text))
        if add_action_button1:
            builder.addAction(
                get_resource("mipmap").icon, cast_object("charSequence", String(action_title1)), pendingIntent
            )

        if add_action_button2:
            builder.addAction(
                get_resource("mipmap").icon, cast_object("charSequence", String(action_title2)), pendingIntent
            )

        if add_action_button3:
            builder.addAction(
                get_resource("mipmap").icon, cast_object("charSequence", String(action_title3)), pendingIntent
            )

        if set_reply:
            builder.addAction(
                NotificationCompatActionBuilder(
                    get_resource("mipmap").icon, String(reply_title), pendingIntent).addRemoteInput(
                    RemoteInputBuilder(String(key_text_reply)).setLabel(
                        String(reply_title)).build()
                ).build()
            ).setAutoCancel(auto_cancel)

        if expandable:
            largePicture = NotificationCompatBigPictureStyle(instantiate=True)
            if isinstance(big_picture, int):
                largePicture.bigPicture(BitmapFactory.decodeResource(activity.getResources(), big_picture))
            elif isinstance(big_picture, str):
                largePicture.bigPicture(BitmapFactory.decodeFile(big_picture))
            else:
                largePicture.bigPicture(BitmapFactory.decodeStream(big_picture))
            builder.setStyle(largePicture)

        if set_large_icon:
            if isinstance(large_icon, int):
                builder.setLargeIcon(BitmapFactory.decodeResource(activity.getResources(), large_icon))
            elif isinstance(large_icon, str):
                builder.setLargeIcon(BitmapFactory.decodeFile(large_icon))
            else:
                builder.setLargeIcon(BitmapFactory.decodeStream(large_icon))

        # notificationId is a unique int for each notification that you must define
        notificationManager.notify(ids, builder.build())
except JavaException as error:
    Logger.error(
        f"{str(error)}\n"
        "Kvdroid: Androidx not Enabled, Cannot Import create_notification\n"
        "Use official kivy plyer module instead or enable androidx in buildozer.spec file"
    )



from jnius import autoclass, JavaException


try:
    autoclass("androidx.core.app.ActivityCompat")
except JavaException as e:
    raise JavaException(
        f"{e}\nadd androidx.appcompat:appcompat:1.4.2 to buildozer.spec file: android.gradle_dependencies")

from .browser.customtabs import *
from .core.app import *
from .core.content import *



from jnius import autoclass
from kvdroid.jclass import _class_call


def StreamConfigurationMap(*args, instantiate: bool = False):
    return _class_call(autoclass("android.hardware.camera2.params.StreamConfigurationMap"), args, instantiate)



from kvdroid.jclass.android.webkit import CookieManager


def get_cookies(site_name: str):
    cookieManager = CookieManager().getInstance()
    return cookieManager.getCookie(site_name)


def launch_url(url: str, color="#FFFFFF", color_scheme="system"):
    from kvdroid.jclass.android.graphics import Color
    from kvdroid.jclass.androidx.browser.customtabs import CustomTabColorSchemeParamsBuilder
    from kvdroid.jclass.androidx.browser.customtabs import CustomTabsIntentBuilder, CustomTabsIntent
    from kvdroid import activity
    from kvdroid.jclass.android.net import Uri
    CustomTabsIntent = CustomTabsIntent()
    schemes = {
        "system": CustomTabsIntent.COLOR_SCHEME_SYSTEM,
        "light": CustomTabsIntent.COLOR_SCHEME_LIGHT,
        "dark": CustomTabsIntent.COLOR_SCHEME_DARK
    }
    color_int = Color().parseColor(color)
    default_color = CustomTabColorSchemeParamsBuilder(instantiate=True).setToolbarColor(color_int).build()
    builder = CustomTabsIntentBuilder(instantiate=True)
    if color_scheme not in schemes:
        builder.setColorScheme(CustomTabsIntent.COLOR_SCHEME_SYSTEM)
    else:
        builder.setColorScheme(schemes[color_scheme])
    builder.setDefaultColorSchemeParams(default_color)
    custom_tabs_intent = builder.build()
    custom_tabs_intent.intent.setPackage("com.android.chrome")
    custom_tabs_intent.launchUrl(activity, Uri().parse(url))



from jnius import autoclass
from kvdroid.jclass import _class_call


def Intent(*args, instantiate: bool = False):
    return _class_call(autoclass("android.content.Intent"), args, instantiate)


def Context(*args, instantiate: bool = False):
    return _class_call(autoclass("android.content.Context"), args, instantiate)


def IntentFilter(*args, instantiate: bool = False):
    return _class_call(autoclass("android.content.IntentFilter"), args, instantiate)


def ContentValues(*args, instantiate: bool = False):
    return _class_call(autoclass('android.content.ContentValues'), args, instantiate)


def ContentUris(*args, instantiate: bool = False):
    return _class_call(autoclass('android.content.ContentUris'), args, instantiate)



from jnius import autoclass
from kvdroid.jclass import _class_call


def TextToSpeech(*args, instantiate: bool = False):
    return _class_call(autoclass('android.speech.tts.TextToSpeech'), args, instantiate)



from jnius import autoclass, JavaException

from kvdroid.jclass import _browserx_except_cls_call


def CustomTabsIntent(*args, instantiate: bool = False):
    return _browserx_except_cls_call(
        "androidx.browser.customtabs.CustomTabsIntent", args, instantiate)


def CustomTabsIntentBuilder(*args, instantiate: bool = False):
    return _browserx_except_cls_call(
        "androidx.browser.customtabs.CustomTabsIntent$Builder", args, instantiate)


def CustomTabColorSchemeParams(*args, instantiate: bool = False):
    return _browserx_except_cls_call(
        "androidx.browser.customtabs.CustomTabColorSchemeParams", args, instantiate)


def CustomTabColorSchemeParamsBuilder(*args, instantiate: bool = False):
    return _browserx_except_cls_call(
        "androidx.browser.customtabs.CustomTabColorSchemeParams$Builder", args, instantiate)



from kvdroid import activity
from kvdroid.jclass.android import ApplicationInfo, PackageManager, ComponentName
from kvdroid.jclass.java import File


def all_packages():
    pkgAppsList = activity.getPackageManager().getInstalledPackages(0)
    package_list = []
    for i in range(pkgAppsList.size()):
        package = pkgAppsList.get(i).applicationInfo
        package_list.append(package.packageName)
    return package_list


def all_main_activities():
    from kvdroid.jclass.android import Intent
    mainIntent = Intent(Intent().ACTION_MAIN)
    mainIntent.addCategory(Intent().CATEGORY_LAUNCHER)
    pkgAppsList = activity.getPackageManager().queryIntentActivities(mainIntent, 0)
    activity_list = []
    for i in range(pkgAppsList.size()):
        package = pkgAppsList.get(i).activityInfo
        activity_list.append({package.packageName: package.name})
    return activity_list


def is_system_package(package):
    pManager = activity.getPackageManager()
    package = pManager.getApplicationInfo(
        package, PackageManager().GET_META_DATA)
    if (package.flags & (ApplicationInfo().FLAG_SYSTEM | ApplicationInfo().FLAG_UPDATED_SYSTEM_APP)) != 0:
        return True
    else:
        return False

    
def is_package_enabled(package):
    pManager = activity.getPackageManager()
    return pManager.getApplicationInfo(package, 0).enabled


def package_source(package):
    installer = activity.getPackageManager().getInstallerPackageName(package)
    if installer == "com.android.vending":
        return "playstore"
    else:
        return "unknown"


def package_info(package):
    pManager = activity.getPackageManager()
    appInfo = pManager.getPackageInfo(package, 0)
    installTime = appInfo.firstInstallTime
    updateTime = appInfo.lastUpdateTime
    versionCode = appInfo.getLongVersionCode()
    versionName = appInfo.versionName
    targetSdkVersion = appInfo.applicationInfo.targetSdkVersion
    minSdkVersion = appInfo.applicationInfo.minSdkVersion
    enabled = is_package_enabled(package)    
    application = pManager.getApplicationInfo(
        package, PackageManager().GET_META_DATA)
    size = File(application.publicSourceDir).length()
    loadLabel = application.loadLabel(pManager)
    loadIcon = application.loadIcon(pManager)
    packageName = application.packageName
    sourceDir = application.sourceDir
    dataDir = application.dataDir
    processName = application.processName
    publicSourceDir = application.publicSourceDir
    sharedLibraryFiles = application.sharedLibraryFiles
    packagePerms = pManager.getPackageInfo(
        packageName, PackageManager().GET_PERMISSIONS)
    requestedPermissions = packagePerms.requestedPermissions
    permissions = []
    if requestedPermissions != None:
        for i in range(len(requestedPermissions)):
            permissions.append(requestedPermissions[i])
    activities = []
    activity_list = pManager.getPackageInfo(
        packageName, PackageManager().GET_ACTIVITIES).activities
    if activity_list:
        for act in activity_list:
            activities.append(act.name)
    infos = {"packageName": packageName,
             "loadLabel": loadLabel,
             "loadIcon": loadIcon,
             "sourceDir": sourceDir,
             "dataDir": dataDir,
             "processName": processName,
             "publicSourceDir": publicSourceDir,
             "sharedLibraryFiles": sharedLibraryFiles,
             "installTime": installTime,
             "updateTime": updateTime,
             "versionName": versionName,
             "versionCode": versionCode,
             "targetSdkVersion": targetSdkVersion,
             "minSdkVersion": minSdkVersion,
             "permissions": permissions,
             "activities": activities,
             "enabled": enabled,
             "size": size
             }
    return infos


def activity_info(package, act):
    pManager = activity.getPackageManager()
    component = ComponentName(package, act)
    activityInfo = activity.getPackageManager().getActivityInfo(
        component, PackageManager().GET_META_DATA)
    loadLabel = activityInfo.loadLabel(pManager)
    loadIcon = activityInfo.loadIcon(pManager)
    infos = {
        "loadLabel": loadLabel,
        "loadIcon": loadIcon
    }
    return infos



from jnius import autoclass
from kvdroid.jclass import _class_call


def Canvas(*args, instantiate: bool = False):
    return _class_call(autoclass("android.graphics.Canvas"), args, instantiate)


def Color(*args, instantiate: bool = False):
    return _class_call(autoclass("android.graphics.Color"), args, instantiate)


def Rect(*args, instantiate: bool = False):
    return _class_call(autoclass('android.graphics.Rect'), args, instantiate)


def Bitmap(*args, instantiate: bool = False):
    return _class_call(autoclass('android.graphics.Bitmap'), args, instantiate)


def BitmapFactory(*args, instantiate: bool = False):
    return _class_call(autoclass('android.graphics.BitmapFactory'), args, instantiate)


def Config(*args, instantiate: bool = False):
    return _class_call(autoclass("android.graphics.Bitmap$Config"), args, instantiate)


def CompressFormat(*args, instantiate: bool = False):
    return _class_call(autoclass("android.graphics.Bitmap$CompressFormat"), args, instantiate)


def Point(*args, instantiate: bool = False):
    return _class_call(autoclass("android.graphics.Point"), args, instantiate)



import contextlib
from time import sleep
from typing import Union
from kvdroid import _convert_color
from jnius import JavaException  # NOQA
from kvdroid import activity
from kvdroid.jclass.java import URL
from kvdroid.jclass.android import (
    Intent,
    Context,
    Environment,
    Request,
    WallpaperManager,
    Color,
    BitmapFactory,
    Rect,
    URLUtil
)
from android.runnable import run_on_ui_thread  # NOQA


def toast(message):
    return activity.toastError(str(message))


def share_text(text, title='Share', chooser=False, app_package=None, call_playstore=True, error_msg=""):
    intent = Intent(Intent().ACTION_SEND)  # a function call that returns a java class call
    from kvdroid.jclass.java import String
    intent.putExtra(Intent().EXTRA_TEXT, String(str(text)))
    intent.setType("text/plain")
    if app_package:
        from kvdroid import packages
        app_package = packages[app_package] if app_package in packages else None
        from jnius import JavaException  # NOQA
        try:
            intent.setPackage(String(app_package))
        except JavaException:
            if call_playstore:
                import webbrowser
                webbrowser.open(f"http://play.google.com/store/apps/details?id={app_package}")
            from kvdroid import Logger
            toast(error_msg) if error_msg else Logger.error("Kvdroid: Specified Application is unavailable")
            return
    from kvdroid import activity
    if chooser:
        chooser = Intent().createChooser(intent, String(title))
        activity.startActivity(chooser)
    else:
        activity.startActivity(intent)


def share_file(path, title='Share', chooser=True, app_package=None, call_playstore=True, error_msg=""):
    from kvdroid.jclass.java import String
    path = str(path)
    from kvdroid.jclass.android import VERSION
    if VERSION().SDK_INT >= 24:
        from kvdroid.jclass.android import StrictMode
        StrictMode().disableDeathOnFileUriExposure()
    shareIntent = Intent(Intent().ACTION_SEND)
    shareIntent.setType("*/*")
    from kvdroid.jclass.java import File
    imageFile = File(path)
    from kvdroid.jclass.android import Uri
    uri = Uri().fromFile(imageFile)
    from kvdroid.cast import cast_object
    parcelable = cast_object('parcelable', uri)
    shareIntent.putExtra(Intent().EXTRA_STREAM, parcelable)

    if app_package:
        from kvdroid import packages
        app_package = packages[app_package] if app_package in packages else None
        from jnius import JavaException
        try:
            shareIntent.setPackage(String(app_package))
        except JavaException:
            if call_playstore:
                import webbrowser
                webbrowser.open(f"http://play.google.com/store/apps/details?id={app_package}")
            from kvdroid import Logger
            toast(error_msg) if error_msg else Logger.error("Kvdroid: Specified Application is unavailable")
            return

    if chooser:
        chooser = Intent().createChooser(shareIntent, String(title))
        activity.startActivity(chooser)
    else:
        activity.startActivity(shareIntent)


def mime_type(file_path):
    from kvdroid.jclass.java import URLConnection
    return URLConnection().guessContentTypeFromName(file_path)


def get_resource(resource, activity_type=activity):
    from jnius import autoclass
    return autoclass(f"{activity_type.getPackageName()}.R${resource}")


def restart_app():
    from kvdroid.cast import cast_object
    currentActivity = cast_object('activity', activity)
    context = cast_object('context', currentActivity.getApplicationContext())
    packageManager = context.getPackageManager()
    intent = packageManager.getLaunchIntentForPackage(context.getPackageName())
    componentName = intent.getComponent()
    mainIntent = Intent().makeRestartActivityTask(componentName)
    context.startActivity(mainIntent)
    from kvdroid.jclass.java.lang import Runtime
    Runtime().getRuntime().exit(0)


def download_manager(title, description, url, folder=None, file_name=None):
    from kvdroid.jclass.android import Uri
    uri = Uri().parse(str(url))
    from kvdroid.cast import cast_object
    dm = cast_object("downloadManager", activity.getSystemService(Context().DOWNLOAD_SERVICE))
    request = Request(uri)
    request.setTitle(str(title))
    request.setDescription(str(description))
    request.setNotificationVisibility(Request().VISIBILITY_VISIBLE_NOTIFY_COMPLETED)
    if folder and file_name:
        request.setDestinationInExternalPublicDir(str(folder), str(file_name))
    else:
        conn = URL(url).openConnection()
        with contextlib.suppress(JavaException):
            conn.getContent()
        url = conn.getURL().toString()
        print(url)
        request.setDestinationInExternalPublicDir(
            Environment().DIRECTORY_DOWNLOADS, URLUtil().guessFileName(url, None, None)
        )
    dm.enqueue(request)


@run_on_ui_thread
def change_statusbar_color(color: Union[str, list], text_color):
    color = _convert_color(color)
    window = activity.getWindow()
    if str(text_color) == "black":
        from kvdroid.jclass.android import View
        window.getDecorView().setSystemUiVisibility(View().SYSTEM_UI_FLAG_LIGHT_STATUS_BAR)
    elif str(text_color) == "white":
        window.getDecorView().setSystemUiVisibility(0)
    else:
        raise TypeError("Available options are ['white','black'] for StatusBar text color")
    from kvdroid.jclass.android import WindowManagerLayoutParams
    window.clearFlags(WindowManagerLayoutParams().FLAG_TRANSLUCENT_STATUS)
    window.addFlags(WindowManagerLayoutParams().FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS)
    window.setStatusBarColor(Color().parseColor(color))


@run_on_ui_thread
def navbar_color(color: Union[str, list]):
    color = _convert_color(color)
    window = activity.getWindow()
    window.setNavigationBarColor(Color().parseColor(color))


def set_wallpaper(path_to_image):
    from kvdroid.cast import cast_object
    context = cast_object('context', activity.getApplicationContext())
    bitmap = BitmapFactory().decodeFile(path_to_image)
    manager = WallpaperManager().getInstance(context)
    return manager.setBitmap(bitmap)


def speech(text: str, lang: str):
    from kvdroid.jclass.android import TextToSpeech
    tts = TextToSpeech(activity, None)
    retries = 0
    from kvdroid.jclass.java import Locale
    tts.setLanguage(Locale(lang))
    speak_status = tts.speak(text, TextToSpeech().QUEUE_FLUSH, None)
    while retries < 100 and speak_status == -1:
        sleep(0.1)
        retries += 1
        speak_status = tts.speak(
            text, TextToSpeech().QUEUE_FLUSH, None
        )
    return speak_status


def keyboard_height():
    try:
        decor_view = activity.getWindow().getDecorView()
        height = activity.getWindowManager().getDefaultDisplay().getHeight()
        decor_view.getWindowVisibleDisplayFrame(Rect)
        return height - Rect().bottom
    except JavaException:
        return 0


@run_on_ui_thread
def immersive_mode(status='enable'):
    window = activity.getWindow()
    from kvdroid.jclass.android import View
    View = View()
    if status == "disable":
        return window.getDecorView().setSystemUiVisibility(
            View.SYSTEM_UI_FLAG_LAYOUT_STABLE
            | View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
            | View.SYSTEM_UI_FLAG_VISIBLE)
    else:
        return window.getDecorView().setSystemUiVisibility(
            View.SYSTEM_UI_FLAG_LAYOUT_STABLE
            | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
            | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
            | View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
            | View.SYSTEM_UI_FLAG_FULLSCREEN
            | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY)


def launch_app_activity(app_package, app_activity):
    intent = Intent(Intent().ACTION_VIEW)
    intent.setClassName(app_package, app_activity)
    return activity.startActivity(intent)


def launch_app(app_package):
    intent = activity.getPackageManager().getLaunchIntentForPackage(app_package)
    activity.startActivity(intent)


def app_details(app_package):
    from kvdroid.jclass.android import Settings
    intent = Intent(Settings().ACTION_APPLICATION_DETAILS_SETTINGS)
    from kvdroid.jclass.android import Uri
    uri = Uri().parse(f"package:{app_package}")
    intent.setData(uri)
    activity.startActivity(intent)



import logging
from os import environ
from typing import Union

from jnius import autoclass # NOQA


def _get_platform():
    # On Android sys.platform returns 'linux2', so prefer to check the
    # existence of environ variables set during Python initialization
    kivy_build = environ.get('KIVY_BUILD', '')
    if kivy_build in {'android', 'ios'}:
        return kivy_build
    elif 'P4A_BOOTSTRAP' in environ or 'ANDROID_ARGUMENT' in environ:
        return 'android'


def get_hex_from_color(color: list):
    return "#" + "".join([f"{i * 255:02x}" for i in color])


def _convert_color(color: Union[str, list]):
    if isinstance(color, list):
        color = get_hex_from_color(color)
    return color


platform = _get_platform()
Logger = logging.getLogger('kivy')

if platform == "android":
    try:
        from android import config # NOQA

        ns = config.JAVA_NAMESPACE
    except (ImportError, AttributeError):
        ns = 'org.renpy.android'

    if 'PYTHON_SERVICE_ARGUMENT' in environ:
        PythonService = autoclass(ns + '.PythonService')
        activity = PythonService.mService
    else:
        PythonActivity = autoclass(ns + '.PythonActivity')
        activity = PythonActivity.mActivity

    packages = {
        "whatsapp": "com.whatsapp",
        "facebook": "com.facebook.katana",
        "facebookLite": "com.facebook.lite",
        "oldFacebook": "com.facebook.android",
        "linkedin": "com.linkedin.android",
        "fbMessenger": "com.facebook.orca",
        "fbMessengerLite": "com.facebook.mlite",
        "tiktok": "com.zhiliaoapp.musically",
        "tiktokLite": "com.zhiliaoapp.musically.go",
        "twitter": "com.twitter.android",
        "twitterLite": "com.twitter.android.lite",
        "telegram": "org.telegram.messenger",
        "telegramX": "org.thunderdog.challegram",
        "snapchat": "com.snapchat.android",
        "chrome": "com.android.chrome"
    }
else:
    Logger.error(
        "Kvdroid: Kvdroid is only callable from Android"
    )



from jnius import autoclass, JavaException


def _class_call(cls, args: tuple, instantiate: bool):
    if not args:
        return cls() if instantiate else cls
    else:
        return cls(*args)


def _browserx_except_cls_call(namespace: str, args: tuple, instantiate: bool):
    try:
        return _class_call(autoclass(namespace), args, instantiate)
    except JavaException as e:
        raise JavaException(
            f"{e}\nEnable androidx in your buildozer.spec file\nadd 'androidx.browser:browser:1.4.0' to "
            f"buildozer.spec file: android.gradle_dependencies"
        )



from jnius import autoclass
from kvdroid.jclass import _class_call


def Configuration(*args, instantiate: bool = False):
    return _class_call(autoclass("android.content.res.Configuration"), args, instantiate)


def Resources(*args, instantiate: bool = False):
    return _class_call(autoclass("android.content.res.Resources"), args, instantiate)



from setuptools import setup, find_packages

from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='kvdroid',
    packages=find_packages(
            include=["kvdroid", "kvdroid.*"]
        ),
    version='0.3.0',
    description='A re-implementation of android java API in python with easy access to some Android functionality '
                'like Notification,Reading of Contacts, accessing Webview Cookies, etc...',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Yunus Ceyhan',
    author_email='yunus.ceyhn@gmail.com',
    url='https://github.com/kvdroid/Kvdroid',
    keywords=['Android', 'Androidx', 'Python', 'Kivy', 'KivyMD', "KvDroid"],
    install_requires=["pyjnius"],
    classifiers=[],
)



from jnius import autoclass
from kvdroid.jclass import _class_call

if autoclass('android.os.Build$VERSION').SDK_INT >= 26:
    def NotificationManagerCompat(*args, instantiate: bool = False):
        return _class_call(
            autoclass("androidx.core.app.NotificationManagerCompat"), args, instantiate)


def NotificationCompat(*args, instantiate: bool = False):
    return _class_call(
        autoclass("androidx.core.app.NotificationCompat"), args, instantiate)


def NotificationCompatBigPictureStyle(*args, instantiate: bool = False):
    return _class_call(
        autoclass("androidx.core.app.NotificationCompat$BigPictureStyle"), args, instantiate)


def NotificationCompatAction(*args, instantiate: bool = False):
    return _class_call(
        autoclass("androidx.core.app.NotificationCompat$Action"), args, instantiate)


def NotificationCompatActionBuilder(*args, instantiate: bool = False):
    return _class_call(
        autoclass("androidx.core.app.NotificationCompat$Action$Builder"), args, instantiate)


def NotificationCompatBuilder(*args, instantiate: bool = False):
    return _class_call(
        autoclass("androidx.core.app.NotificationCompat$Builder"), args, instantiate)


def NotificationBigTextStyle(*args, instantiate: bool = False):
    return _class_call(
        autoclass("androidx.core.app.NotificationCompat$BigTextStyle"), args, instantiate)


def RemoteInput(*args, instantiate: bool = False):
    return _class_call(
        autoclass("androidx.core.app.RemoteInput"), args, instantiate)


def RemoteInputBuilder(*args, instantiate: bool = False):
    return _class_call(
        autoclass("androidx.core.app.RemoteInput$Builder"), args, instantiate)



import os

from kvdroid.jclass.android.os import Environment


def path():
    return os.path.dirname(os.path.abspath("__main__"))


def sdcard():
    return Environment().getExternalStorageDirectory().toString()


def external_sdcard():
    try:
        return os.path.join("/storage", os.listdir("/storage")[1])
    except Exception:
        return None



from jnius import autoclass
from kvdroid.jclass import _class_call


def BitmapDrawable(*args, instantiate: bool = False):
    return _class_call(autoclass("android.graphics.drawable.BitmapDrawable"), args, instantiate)


def Drawable(*args, instantiate: bool = False):
    return _class_call(autoclass("android.graphics.drawable.Drawable"), args, instantiate)


def AdaptiveIconDrawable(*args, instantiate: bool = False):
    return _class_call(autoclass('android.graphics.drawable.AdaptiveIconDrawable'), args, instantiate)



from jnius import autoclass
from kvdroid.jclass import _class_call


def CookieManager(*args, instantiate: bool = False):
    return _class_call(autoclass("android.webkit.CookieManager"), args, instantiate)


def URLUtil(*args, instantiate: bool = False):
    return _class_call(autoclass("android.webkit.URLUtil"), args, instantiate)


def MimeTypeMap(*args, instantiate: bool = False):
    return _class_call(autoclass('android.webkit.MimeTypeMap'), args, instantiate)



from jnius import autoclass
from kvdroid.jclass import _class_call


def Settings(*args, instantiate: bool = False):
    return _class_call(autoclass('android.provider.Settings'), args, instantiate)


def Contacts(*args, instantiate: bool = False):
    return _class_call(autoclass('android.provider.ContactsContract$Contacts'), args, instantiate)


def Phone(*args, instantiate: bool = False):
    return _class_call(
        autoclass('android.provider.ContactsContract$CommonDataKinds$Phone'),
        args, instantiate)


def MediaStoreFiles(*args, instantiate: bool = False):
    return _class_call(autoclass('android.provider.MediaStore$Files'), args, instantiate)


def MediaStoreAudioMedia(*args, instantiate: bool = False):
    return _class_call(autoclass('android.provider.MediaStore$Audio$Media'), args, instantiate)


def MediaStoreImagesMedia(*args, instantiate: bool = False):
    return _class_call(autoclass('android.provider.MediaStore$Images$Media'), args, instantiate)


def MediaStoreVideoMedia(*args, instantiate: bool = False):
    return _class_call(autoclass('android.provider.MediaStore$Video$Media'), args, instantiate)


def MediaStoreDownloads(*args, instantiate: bool = False):
    return _class_call(autoclass('android.provider.MediaStore$Downloads'), args, instantiate)


def MediaStoreMediaColumns(*args, instantiate: bool = False):
    return _class_call(autoclass('android.provider.MediaStore$MediaColumns'), args, instantiate)


def TelephonySms(*args, instantiate: bool = False):
    return _class_call(autoclass('android.provider.Telephony$Sms'), args, instantiate)


def CallLogCalls(*args, instantiate: bool = False):
    return _class_call(autoclass('android.provider.CallLog$Calls'), args, instantiate)



from jnius import autoclass
from kvdroid.jclass import _class_call


def Formatter(*args, instantiate: bool = False):
    return _class_call(autoclass('android.text.format.Formatter'), args, instantiate)



from .content.res import *
from .content import *
from .speech.tts import *
from .speech import *
from .app import *
from .graphics import *
from .graphics.drawable import *
from .media import *
from .net import *
from .os import *
from .provider import *
from .view import *
from .webkit import *
from .widget import *
from .net.wifi import *
from .text.format import *
from .hardware.camera2 import *
from .hardware.camera2.params import *
from .support.v4.app import *
from .content.pm import *
from .R import *

from jnius import autoclass
from kvdroid.jclass import _class_call


def Manifest(*args, instantiate: bool = False):
    return _class_call(autoclass("android.Manifest"), args, instantiate)





from jnius import autoclass # NOQA
from kvdroid.jclass import _class_call


def NotificationManager(*args, instantiate: bool = False):
    return _class_call(autoclass("android.app.NotificationManager"), args, instantiate)


def Notification(*args, instantiate: bool = False):
    return _class_call(autoclass("android.app.Notification"), args, instantiate)


if autoclass('android.os.Build$VERSION').SDK_INT >= 26:
    def NotificationChannel(*args, instantiate: bool = False):
        return _class_call(autoclass("android.app.NotificationChannel"), args, instantiate)


def WallpaperManager(*args, instantiate: bool = False):
    return _class_call(autoclass('android.app.WallpaperManager'), args, instantiate)


def Request(*args, instantiate: bool = False):
    return _class_call(autoclass("android.app.DownloadManager$Request"), args, instantiate)


def Activity(*args, instantiate: bool = False):
    return _class_call(autoclass('android.app.Activity'), args, instantiate)


def PendingIntent(*args, instantiate: bool = False):
    return _class_call(autoclass("android.app.PendingIntent"), args, instantiate)


def MemoryInfo(*args, instantiate: bool = False):
    return _class_call(autoclass('android.app.ActivityManager$MemoryInfo'), args, instantiate)


def ActivityManager(*args, instantiate: bool = False):
    return _class_call(autoclass("android.app.ActivityManager"), args, instantiate)

    
def ComponentName(*args, instantiate: bool = False):
    return _class_call(autoclass("android.content.ComponentName"), args, instantiate)

    
def ApplicationInfo(*args, instantiate: bool = False):
    return _class_call(autoclass("android.content.pm.ApplicationInfo"), args, instantiate)

    
def PackageManager(*args, instantiate: bool = False):
    return _class_call(autoclass("android.content.pm.PackageManager"), args, instantiate)

    
def Configuration(*args, instantiate: bool = False):
    return _class_call(autoclass("android.content.res.Configuration"), args, instantiate)



from jnius import autoclass
from kvdroid.jclass import _class_call


def ContextCompat(*args, instantiate: bool = False):
    return _class_call(
        autoclass("androidx.core.content.ContextCompat"), args, instantiate)


def FileProvider(*args, instantiate: bool = False):
    return _class_call(
        autoclass("androidx.core.content.FileProvider"), args, instantiate)



from jnius import autoclass
from kvdroid.jclass import _class_call


def View(*args, instantiate: bool = False):
    return _class_call(autoclass('android.view.View'), args, instantiate)


def WindowManager(*args, instantiate: bool = False):
    return _class_call(autoclass("android.view.WindowManager"), args, instantiate)


def WindowManagerLayoutParams(*args, instantiate: bool = False):
    return _class_call(autoclass("android.view.WindowManager$LayoutParams"), args, instantiate)


def ViewGroupLayoutParams(*args, instantiate: bool = False):
    return _class_call(autoclass('android.view.ViewGroup$LayoutParams'), args, instantiate)
